package com.company.bugy.view.testentity;

import com.company.bugy.entity.TestEntity;

import com.company.bugy.view.main.MainView;

import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.router.Route;
import io.jmix.core.DataManager;
import io.jmix.flowui.UiComponents;
import io.jmix.flowui.component.textfield.TypedTextField;
import io.jmix.flowui.data.value.ContainerValueSource;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.model.DataComponents;
import io.jmix.flowui.model.InstanceContainer;
import io.jmix.flowui.view.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.MathContext;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

@Route(value = "testEntities/:id", layout = MainView.class)
@ViewController("bugy_TestEntity.detail")
@ViewDescriptor("test-entity-detail-view.xml")
@EditedEntityContainer("testEntityDc")
public class TestEntityDetailView extends StandardDetailView<TestEntity> {
    @Autowired
    private UiComponents uiComponents;
    @ViewComponent
    private InstanceContainer<TestEntity> testEntityDc;
    @Autowired
    private DataManager dataManager;
    @ViewComponent
    private HorizontalLayout hboxContainer;

    @Subscribe("calculate")
    public void onCalculateClick(final ClickEvent<JmixButton> event) throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        calculate();
    }

    @Subscribe
    public void onReady(final ReadyEvent event) {
        generateTextfields();
    }
    private void generateTextfields(){
        for (int i=1 ; i<= 5; i++) {
            String zName =  StringUtils.leftPad(String.valueOf(i), 2, '0');
            TypedTextField textField = uiComponents.create(TypedTextField.class);

            textField.setValueSource(new ContainerValueSource<>(testEntityDc, "z" + zName));

            textField.addValueChangeListener(textChangeEvent -> {
                try {

                    calculate();
                } catch (NoSuchMethodException e) {
                    throw new RuntimeException(e);
                } catch (InvocationTargetException e) {
                    throw new RuntimeException(e);
                } catch (IllegalAccessException e) {
                    throw new RuntimeException(e);
                }
            });
            hboxContainer.add(textField);
        }
    }

    private void calculate() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        BigDecimal total;
        total = BigDecimal.ZERO;

        for (int i = 1; i <= 5; i++) {
            String zName = StringUtils.leftPad(String.valueOf(i), 2, '0');
            Method method = testEntityDc.getItem().getClass().getMethod("getZ" + zName);
            String zReturn = (String) method.invoke(testEntityDc.getItem());
            BigDecimal rezult = stringToBigDecimal(zReturn);
            total = total.add(rezult);
        }
        testEntityDc.getItem().setTotal(total);

    }

    public static BigDecimal stringToBigDecimal(String cString)  {
        // Create a DecimalFormat that fits your requirements
        DecimalFormatSymbols symbols = new DecimalFormatSymbols();
        symbols.setGroupingSeparator(',');
        symbols.setDecimalSeparator('.');
        String pattern = "#,##0.0#";
        DecimalFormat decimalFormat = new DecimalFormat(pattern, symbols);
        decimalFormat.setParseBigDecimal(true);

        // parse the string
        BigDecimal bigDecimal = BigDecimal.ZERO;
        try {
            bigDecimal = (BigDecimal) decimalFormat.parse(cString);
        } catch (Exception e){
            bigDecimal = BigDecimal.ZERO;
        }

        return bigDecimal;
    }

}